/* $Id: rpmver.c,v 1.6 2005/04/19 00:20:57 nanardon Exp $ */
#include <postgres.h>
#include <fmgr.h>

#include <rpm/rpmlib.h>

#include "pgrpm.h"
#include "rpmver.h"

static
void parseEVR(char * evr,
		/*@exposed@*/ /*@out@*/ const char ** ep,
		/*@exposed@*/ /*@out@*/ const char ** vp,
		/*@exposed@*/ /*@out@*/ const char ** rp)
	/*@modifies *ep, *vp, *rp @*/
	/*@requires maxSet(ep) >= 0 /\ maxSet(vp) >= 0 /\ maxSet(rp) >= 0 @*/
{
    const char *epoch;
    const char *version;		/* assume only version is present */
    const char *release;
    char *s, *se;

    s = evr;
    while (*s && xisdigit(*s)) s++;	/* s points to epoch terminator */
    se = strrchr(s, '-');		/* se points to version terminator */

    if (*s == ':') {
	epoch = evr;
	*s++ = '\0';
	version = s;
	/*@-branchstate@*/
	if (*epoch == '\0') epoch = "0";
	/*@=branchstate@*/
    } else {
	epoch = NULL;	/* XXX disable epoch compare if missing */
	version = evr;
    }
    if (se) {
/*@-boundswrite@*/
	*se++ = '\0';
/*@=boundswrite@*/
	release = se;
    } else {
	release = NULL;
    }

    if (ep) *ep = epoch;
    if (vp) *vp = version;
    if (rp) *rp = release;
}

static int compare_values(const char *str1, const char *str2)
{
    if (!str1 && !str2)
    return 0;
    else if (str1 && !str2)
    return 1;
    else if (!str1 && str2)
    return -1;
    return rpmvercmp(str1, str2);
}

static int
_pgrpmvercmp(text * t1, text * t2) {
    char *aevr = GET_STR(t1);
    char *bevr = GET_STR(t2);
    int rc = 0;

    const char * ae, * av, * ar;
    const char * be, * bv, * br;

    parseEVR(aevr, &ae, &av, &ar);
    parseEVR(bevr, &be, &bv, &br);

    rc = compare_values(ae, be);
    if (!rc) {
        rc = compare_values(av, bv);
        if (!rc)
            rc = compare_values(ar, br);
    }
#ifdef FREE_OURSELF    
    pfree(aevr);
    pfree(bevr);
#endif
    return rc;
}

PG_FUNCTION_INFO_V1(pgrpmvercmp);
Datum
pgrpmvercmp(PG_FUNCTION_ARGS) {
    PG_RETURN_INT32(_pgrpmvercmp(PG_GETARG_TEXT_P(0), PG_GETARG_TEXT_P(1)));
}

PG_FUNCTION_INFO_V1(pgrpmvercmpeq);
Datum
pgrpmvercmpeq(PG_FUNCTION_ARGS) {
    if(_pgrpmvercmp(PG_GETARG_TEXT_P(0), PG_GETARG_TEXT_P(1)) == 0)
        PG_RETURN_BOOL(TRUE);
    else
        PG_RETURN_BOOL(FALSE);
}

PG_FUNCTION_INFO_V1(pgrpmvercmpne);
Datum
pgrpmvercmpne(PG_FUNCTION_ARGS) {
    if(_pgrpmvercmp(PG_GETARG_TEXT_P(0), PG_GETARG_TEXT_P(1)) == 0)
        PG_RETURN_BOOL(FALSE);
    else
        PG_RETURN_BOOL(TRUE);
}

PG_FUNCTION_INFO_V1(pgrpmvercmpgt);
Datum
pgrpmvercmpgt(PG_FUNCTION_ARGS) {
    if(_pgrpmvercmp(PG_GETARG_TEXT_P(0), PG_GETARG_TEXT_P(1)) == 1)
        PG_RETURN_BOOL(TRUE);
    else
        PG_RETURN_BOOL(FALSE);
}

PG_FUNCTION_INFO_V1(pgrpmvercmpget);
Datum
pgrpmvercmpget(PG_FUNCTION_ARGS) {
    if(_pgrpmvercmp(PG_GETARG_TEXT_P(0), PG_GETARG_TEXT_P(1)) == -1)
        PG_RETURN_BOOL(FALSE);
    else
        PG_RETURN_BOOL(TRUE);
}

PG_FUNCTION_INFO_V1(pgrpmvercmplt);
Datum
pgrpmvercmplt(PG_FUNCTION_ARGS) {
    if(_pgrpmvercmp(PG_GETARG_TEXT_P(0), PG_GETARG_TEXT_P(1)) == -1)
        PG_RETURN_BOOL(TRUE);
    else
        PG_RETURN_BOOL(FALSE);
}

PG_FUNCTION_INFO_V1(pgrpmvercmplet);
Datum
pgrpmvercmplet(PG_FUNCTION_ARGS) {
    if(_pgrpmvercmp(PG_GETARG_TEXT_P(0), PG_GETARG_TEXT_P(1)) == 1)
        PG_RETURN_BOOL(FALSE);
    else
        PG_RETURN_BOOL(TRUE);
}

PG_FUNCTION_INFO_V1(pgrpmvercmpmin);
Datum
pgrpmvercmpmin(PG_FUNCTION_ARGS) {
    if (PG_ARGISNULL(0)) PG_RETURN_TEXT_P(PG_GETARG_TEXT_P(1));
    if (PG_ARGISNULL(1)) PG_RETURN_NULL();
    if (_pgrpmvercmp(PG_GETARG_TEXT_P(0), PG_GETARG_TEXT_P(1)) == -1)
        PG_RETURN_TEXT_P(PG_GETARG_TEXT_P(0));
    else
        PG_RETURN_TEXT_P(PG_GETARG_TEXT_P(1));
}

PG_FUNCTION_INFO_V1(pgrpmvercmpmax);
Datum
pgrpmvercmpmax(PG_FUNCTION_ARGS) {
    if (PG_ARGISNULL(0)) PG_RETURN_TEXT_P(PG_GETARG_TEXT_P(1));
    if (PG_ARGISNULL(1)) PG_RETURN_NULL();
    if (_pgrpmvercmp(PG_GETARG_TEXT_P(0), PG_GETARG_TEXT_P(1)) == 1)
        PG_RETURN_TEXT_P(PG_GETARG_TEXT_P(0));
    else
        PG_RETURN_TEXT_P(PG_GETARG_TEXT_P(1));
}
